from luna.baseLunaUtils import BaseLunaUtils
import os

class LocalLunaUtils(BaseLunaUtils):
    def RegisterModel(self, model_path, description, luna_python_model=None):
        # do nothing
        return
    
    def GetModelPath(self, context):
        return os.getcwd()

    
    def DeployModel(self):
        return
        
    def DownloadModel(self, model_path="models"):
        return os.path.join(os.getcwd(), model_path)


from luna.baseLunaUtils import BaseLunaUtils
from mlflow.pyfunc import PythonModel, PythonModelContext
from mlflow.tracking.client import MlflowClient

import mlflow
import os
import yaml

MLFLOW_MODEL_PATH = "MLFLOW_MODELS"

class MLflowLunaUtils(BaseLunaUtils):

    def Init(self, luna_config, run_mode, args, userInput):
        super().Init(luna_config, run_mode, args, userInput)
        mlflow.set_tracking_uri('databricks')
        if not mlflow.active_run():
            with open(self._luna_config["mlflow"]["test_experiment"]) as file:
                test_exp = yaml.full_load(file)
                mlflow.start_run(experiment_id=test_exp["experiment_id"])

    def RegisterModel(self, model_path, description, luna_python_model=None):
        mlFlowRun = mlflow.active_run()
        if mlFlowRun:
            tags={'userId': self._args.userId, 
                        'productName': self._args.productName, 
                        'deploymentName': self._args.deploymentName, 
                        'apiVersion':self._args.apiVersion,
                        'subscriptionId':self._args.subscriptionId, 
                        'modelId': self._args.modelId}
            mlflow.set_tags(tags)
            mlflow.pyfunc.log_model(artifact_path=model_path, 
            python_model=luna_python_model, 
            artifacts={MLFLOW_MODEL_PATH: model_path}, 
            conda_env=self._luna_config["conda_env"])
            model_uri = "runs:/{run_id}/{artifact_path}".format(run_id=mlFlowRun.info.run_id, artifact_path=model_path)
            mlflow.register_model(
                model_uri,
                self._args.modelId
            )

    def GetModelPath(self, context):
        return context[MLFLOW_MODEL_PATH]

    def DeployModel(self):
        return
        
    def DownloadModel(self, model_path=""):
        currentRun = mlflow.active_run()
        filter_string = 'tags."modelId" = "{model_id}"'.format(model_id = self._args.modelId)
        client = MlflowClient(tracking_uri='databricks')
        runs = client.search_runs(experiment_ids=currentRun.info.experiment_id, filter_string=filter_string)
        
        target_model_path = os.path.join(os.getcwd(), model_path)

        os.makedirs(target_model_path, exist_ok=True)

        full_model_path = client.download_artifacts(runs[0].info.run_id, "models/artifacts/", target_model_path)
        return full_model_path


from azureml.core import Workspace, Run, Experiment
from azureml.core.compute import ComputeTarget, AmlCompute
from azureml.core.environment import Environment
from azureml.core.model import Model, InferenceConfig
from azureml.core.runconfig import RunConfiguration
from azureml.core.webservice import AksWebservice, AciWebservice, Webservice
from azureml.pipeline.core.graph import PipelineParameter
from azureml.pipeline.steps import PythonScriptStep
from azureml.pipeline.core import Pipeline

from luna.baseLunaUtils import BaseLunaUtils

import os
import yaml

class AzureMLLunaUtils(BaseLunaUtils):

    def GetAMLWorkspace(self):
        try:
            run = Run.get_context(allow_offline=False)
            return run.experiment.workspace
        except:
            return Workspace.from_config(path=self._luna_config["azureml"]["test_workspace_path"], 
                _file_name=self._luna_config["azureml"]["test_workspace_file_name"])
    
    def RegisterModel(self, model_path, description, luna_python_model=None):
        ws = self.GetAMLWorkspace()

        Model.register(model_path = model_path,
                       model_name = self._args.modelId,
                       description = description,
                       workspace = ws,
                       tags={'userId': self._args.userId, 
                        'productName': self._args.productName, 
                        'deploymentName': self._args.deploymentName, 
                        'apiVersion':self._args.apiVersion,
                        'subscriptionId':self._args.subscriptionId,
                        'modelId': self._args.modelId})

    def GetDeploymentConfig(self, tags):
        with open(self._luna_config['azureml']['workspace_config']) as file:
            documents = yaml.full_load(file)
            deployment_target = documents['deployment_target']
            if deployment_target == 'aks':
                aks_cluster = documents['aks_cluster']


        with open(self._luna_config['deploy_config']) as file:
            documents = yaml.full_load(file)

            if deployment_target == 'aci':
                deployment_config = AciWebservice.deploy_configuration()
                deployment_config.__dict__.update(documents['azureContainerInstance'])
                deployment_config.dns_name_label = self._userInput["dns_name_label"]
            elif deployment_target == 'aks':
                deployment_config = AksWebservice.deploy_configuration()
                deployment_config.__dict__.update(documents['kubernetes'])
                deployment_config.compute_target_name = aks_cluster
                deployment_config.namespace = self._userInput["dns_name_label"]

            deployment_config.tags = tags
        return deployment_config

    def DeployModel(self):
        
        ws = self.GetAMLWorkspace()
        model = Model(ws, self._args.modelId)

        print(model)

        myenv = Environment.from_conda_specification('scoring', self.luna_config['conda_env'])

        print(self.luna_config['code']['score'])
        print(os.getcwd())

        inference_config = InferenceConfig(entry_script=self.luna_config['code']['score'], source_directory = os.getcwd(), environment=myenv)

        deployment_config = self.GetDeploymentConfig(
            tags={'userId': self._args.userId, 
                'productName': self._args.productName, 
                'deploymentName': self._args.deploymentName, 
                'apiVersion':self._args.apiVersion,
                'subscriptionId':self._args.subscriptionId,
                'modelId': self._args.modelId})
        
        service = Model.deploy(ws, self._args.endpointId, [model], inference_config, deployment_config)
        service.wait_for_deployment(show_output = True)

    def DownloadModel(self, model_path=""):
        ws = self.GetAMLWorkspace()
        model = Model(ws, self._args.modelId)
        full_model_path = os.path.join(os.getcwd(), model_path, "models/artifacts")
        
        os.makedirs(full_model_path, exist_ok=True)
        model.download(target_dir = full_model_path, exist_ok=True)
        return full_model_path